package LastLyX;
# This package is the last step in creating a LyX file. It:
#  - rejoins the lyx preamble to the rest of the file
#  - prints out the "header" information for tables
#  - translates any remaining unknown commands (well, not yet -Ak 980131)
#  - creates a '.lyx' file

use strict;
use RelyxTable; # handle LaTeX tables

sub last_lyx {
# Arg0 is input file name
# Arg1 is output file name (foo.lyx)
# Arg2 is a string containing the entire preamble

    my ($InFileName, $OutFileName, $LyXPreamble) = (shift, shift, shift);
    warn "Writing LyX file $OutFileName... \n";
    open (INFILE, "<$InFileName");
    open (OUTFILE, ">$OutFileName");

    # Print the preamble
    print OUTFILE $LyXPreamble;

    # Now print out the rest of the LyX file
    # If we need to, write out the header information for a table.
    #    Otherwise just print all lines as they appear
    # In the future, we could buffer text, and then get rid of extra
    #     '\latex default \latex latex' or '\end_deeper \begin_deeper' pieces
    #     created by the translator
    while (<INFILE>) {
	if (/$RelyxTable::TableBeginString/o) {
	    print OUTFILE &print_table;
	} else {
	    print OUTFILE $_;
	}
    }

    close INFILE; close OUTFILE;
    warn "Done writing LyX file!\n";
} # end sub last_lyx

sub print_table {
# Print a table, from TableBeginString to TableEndString
# Also (kind of a hack) remove the last \newline in a table, if any,
#    since it causes LyX to seg fault.
    my $to_print; # string to collect the table in
    my $thistable = shift(@RelyxTable::table_array);

    # First, it needs the header information
    $to_print = $thistable->print_info;

    # Next, print out the whole table
    my $line;
    while (($line = <INFILE>) !~ /$RelyxTable::TableEndString/o) {
        $to_print .= $line;
    }

    # Finally, remove the last \newline, if any
    $to_print =~ s/\\newline(?=\s*)$//;

    return $to_print;
} # end sub print_table

1; # return true to main package
