/* This file is part of
* ======================================================
* 
*           LyX, The Document Processor
*
*	    Copyright (C) 1995 Matthias Ettrich
*           Copyright (C) 1995-1998 The LyX Team.
*
*           This file is Copyright 1996-1998
*           Lars Gullik Bjnnes
*
*======================================================*/

//  Added pseudo-action handling, asierra 180296

#include <config.h>

#ifdef __GNUG__
#pragma implementation
#endif

#include "toolbarbackend.h"
#include "error.h"

// 	$Id: toolbarbackend.C,v 1.1.1.1 1998/04/23 16:02:59 larsbj Exp $	

#if !defined(lint) && !defined(WITH_WARNINGS)
static char vcid[] = "$Id: toolbarbackend.C,v 1.1.1.1 1998/04/23 16:02:59 larsbj Exp $";
#endif /* lint */

extern LyXAction lyxaction;

// enum tooltags_ {
// 	TO_ADD = 1,
// 	TO_ENDTOOLBAR,
//         TO_SEPARATOR,
//         TO_LAYOUTS,
//         TO_NEWLINE,
// 	TO_LAST
// };


// struct keyword_item toolTags[TO_LAST-1] = {
// 	{ "\\add", TO_ADD },
// 	{ "\\end_toolbar", TO_ENDTOOLBAR },
//         { "\\layouts", TO_LAYOUTS },
//         { "\\newline", TO_NEWLINE },
//        { "\\separator", TO_SEPARATOR }
// };


void ToolbarBackend::init()
{
	add(TOOL_LAYOUTS);
	add(LFUN_MENUOPEN);
	//add(LFUN_CLOSEBUFFER);
	add(LFUN_MENUWRITE);
	add(LFUN_MENUPRINT);
	add(TOOL_SEPARATOR);

	add(LFUN_CUT);
	add(LFUN_COPY);
	add(LFUN_PASTE);
	add(TOOL_SEPARATOR);
	
	add(LFUN_EMPH);
	add(LFUN_NOUN);
	add(LFUN_FREE);
	add(TOOL_SEPARATOR);
	
	add(LFUN_FOOTMELT);
	add(LFUN_MARGINMELT);
	add(LFUN_DEPTH);
	add(TOOL_SEPARATOR);

	add(LFUN_TEX);
        add(LFUN_MATH_MODE);
	add(TOOL_SEPARATOR);

	add(LFUN_FIGURE);
	add(LFUN_TABLE);
	//add(LFUN_MELT);
}


#if 0
// should this be in backend or not?
char **Toolbar::getPixmap(kb_action action, LString const & arg)
{
	char **pixmap = unknown_xpm;
	switch(action){
	case LFUN_MENUOPEN:    pixmap = open_xpm; break;
	case LFUN_CLOSEBUFFER: pixmap = close_xpm; break;
	case LFUN_MENUPRINT:   pixmap = print1_xpm; break;
	case LFUN_MENUWRITE:   pixmap = save_xpm; break;
	case LFUN_EMPH:	 pixmap = emph_xpm; break;
	case LFUN_NOUN:        pixmap = noun_xpm; break;
	case LFUN_FREE:        pixmap = free_xpm; break;
	case LFUN_FOOTMELT:    pixmap = foot_xpm; break;
	case LFUN_DEPTH:       pixmap = depth_xpm; break;
	case LFUN_COPY:        pixmap = copy_xpm; break;
	case LFUN_CUT:         pixmap = cut_xpm; break;
	case LFUN_PASTE:       pixmap = paste_xpm; break;
	case LFUN_TEX:         pixmap = tex_xpm; break;
	case LFUN_MATH_MODE:   pixmap = math_xpm; break;
	case LFUN_MARGINMELT:  pixmap = margin_xpm; break;
	case LFUN_FIGURE:      pixmap = fig_xpm; break;
	case LFUN_TABLE:       pixmap = tab_xpm; break;
	case LFUN_MELT:        pixmap = melt_xpm; break;
	case LFUN_QUIT:        pixmap = quit_xpm; break;
	case LFUN_EXPORT:
	{
		if (arg == "ascii")
			pixmap = make_ascii_xpm;
		else if (arg == "latex")
			pixmap = make_latex_xpm;
	}
	break; 
	case LFUN_BOLD : pixmap = bold_xpm; break; 
	case LFUN_SANS: pixmap = sans_xpm; break; 
	case LFUN_RUNLATEX: pixmap = run_latex_xpm; break; 
	case LFUN_PREVIEWPS: pixmap = view_ps_xpm; break; 
	case LFUN_PREVIEW: pixmap = view_dvi_xpm; break; 
	case LFUN_INSERT_MATH:
	{
		if (!arg.empty())
			pixmap = get_pixmap_from_symbol(arg.c_str(),
							buttonwidth,
							height);
	}
	break;
	default:
		//pixmap = unknown_xpm;
		break;
	}
	return pixmap;
}
#endif

void ToolbarBackend::add(int action, bool /*doclean*/)
{
	// adds an item to the list
	toolbarFunc *newItem = 0;
	toolbarFunc *tmp = 0;
	if (action == TOOL_SEPARATOR
	    || action == TOOL_LAYOUTS) {
		newItem = new toolbarFunc;
		newItem->action = action;
		//newItem->help = help;
		// the new item is placed at the end of the list
		tmp = toollist;
		if (tmp != NULL){
			while(tmp->next != NULL)
				tmp = tmp->next;
			// here is tmp->next == NULL
			tmp->next = newItem;
		} else
			toollist = newItem;
	}
}


void ToolbarBackend::clean()
{

}


void ToolbarBackend::read(LyXLex &lex)
{
	//consistency check
	if (lex.GetString() != "\\begin_toolbar")
		lyxerr.print("Toolbar::read: ERROR wrong token:`"
				+lex.GetString()+'\''); 

	struct keyword_item toolTags[TO_LAST-1] = {
		{ "\\add", TO_ADD },
		{ "\\end_toolbar", TO_ENDTOOLBAR },
		{ "\\layouts", TO_LAYOUTS },
		{ "\\newline", TO_NEWLINE },
		{ "\\separator", TO_SEPARATOR }
	};

	clean();
	LString func;
	bool quit = false;
	
	lex.pushTable(toolTags, TO_LAST - 1);

	if (lyxerr.debugging(Error::PARSER))
		lex.printTable();
	
	while (lex.IsOK() && !quit) {
		
		lyxerr.debug("Toolbar::read: current lex text: `"
				+lex.GetString()+'\'',Error::TOOLBAR);

		switch(lex.lex()) {
		  case TO_ADD:
			  if (lex.EatLine()) {
				  func = lex.GetString();
				  lyxerr.debug("Toolbar::read TO_ADD func: `"
					       + func + "'", Error::TOOLBAR);
				  add(lyxaction.LookupFunc(func));
			  }
			  break;
		   
		  case TO_SEPARATOR:
			  add(TOOL_SEPARATOR);
			  break;
		   
		  case TO_LAYOUTS:
			  add(TOOL_LAYOUTS);
			  break;
		   
		  case TO_NEWLINE:
			  add(TOOL_NEWLINE);
			  break;
			
		  case TO_ENDTOOLBAR:
			  // should not set automatically
			  //set();
			  quit = true;
			  break;
		  default:
			  lex.printError("Toolbar::read: "
					  "Unknown toolbar tag: `$$Token'");
			  break;
		}
	}
	lex.popTable();
}
